
#include <irdb-core>
#include <irdb-util>
#include <map>
#include <set>
#include <memory>


namespace IRDB_SDK
{
	using namespace std;

	using AnalysisEngine_t = enum AnalysisEngine
		{
			aeSTARS,
			aeMoon,
			aeRida
		};

	using RegisterIDSet_t    = set<RegisterID_t>;
	using DeadRegisterMap_t  = map<Instruction_t*,RegisterIDSet_t>;
	using StaticGlobalStartMap_t  = map<Instruction_t*, VirtualOffset_t>;
	using RangeSentinelSet_t = InstructionSet_t;

	class DeepAnalysis_t
	{
		protected: 
			DeepAnalysis_t() {}
			DeepAnalysis_t(const DeepAnalysis_t& copy) = delete;

		public:
			virtual unique_ptr<FunctionSet_t          > getLeafFunctions()      const = 0;
			virtual unique_ptr<DeadRegisterMap_t      > getDeadRegisters()      const = 0;
			virtual unique_ptr<StaticGlobalStartMap_t > getStaticGlobalRanges() const = 0;
			virtual unique_ptr<RangeSentinelSet_t     > getRangeSentinels()     const = 0;

		// factories
			static unique_ptr<DeepAnalysis_t> factory(FileIR_t* firp, const AnalysisEngine_t& ae=aeSTARS, const vector<string>& options={});
	};
}
